@@ -64,22 +64,22 @@ static esl_exception_handler_f esl_exception_handler = NULL;
 void
 esl_fail(char *errbuf, const char *format, ...)
 {
-  if (format) {
-    va_list ap;
-    /* Check whether we are running as a daemon so we can do the right thing about logging instead of printing errors */
-    int parent_pid;
-    parent_pid = getppid();
-
-    if(parent_pid != 1){ // we aren't running as a daemon, so print the error normally
-
-    va_start(ap, format);
-    if (errbuf) vsnprintf(errbuf, eslERRBUFSIZE, format, ap);
-    va_end(ap);
+  if (format)
+    {
+      va_list ap;
+      /* Check whether we are running as a daemon so we can do the
+       * right thing about logging instead of printing errors 
+       */
+      if (getppid() != 1)
+	{ // we aren't running as a daemon, so print the error normally
+	  va_start(ap, format);
+	  if (errbuf) vsnprintf(errbuf, eslERRBUFSIZE, format, ap);
+	  va_end(ap);
+	}
+      else vsyslog(LOG_ERR, format, ap); // SRE: TODO: check this.
+                                         // looks wrong. I think it needs va_start(), va_end().
+                                         // also see two more occurrences, below.
     }
-    else { // we are, so log the error instead of printing it
-       vsyslog(LOG_ERR, format, ap);
-    }
-  }
 }
 
 
@@ -143,21 +143,18 @@ esl_exception(int errcode, int use_errno, char *sourcefile, int sourceline, char
     } 
   else 
     {
-        /* Check whether we are running as a daemon so we can do the right thing about logging instead of printing errors */
-      int parent_pid;
-      parent_pid = getppid();
-      if(parent_pid != 1){ // we're not running as a daemon, so print the error normally
-        fprintf(stderr, "Fatal exception (source file %s, line %d):\n", sourcefile, sourceline);
-        va_start(argp, format);
-        vfprintf(stderr, format, argp);
-        va_end(argp);
-        fprintf(stderr, "\n");
-        if (use_errno && errno) perror("system error");
-        fflush(stderr);
-      }  
-      else { // we are, so log the error instead of printing it
-        vsyslog(LOG_ERR, format, argp);
-      }
+      /* Check whether we are running as a daemon so we can do the right thing about logging instead of printing errors */
+      if (getppid() != 1)
+	{ // we're not running as a daemon, so print the error normally
+	  fprintf(stderr, "Fatal exception (source file %s, line %d):\n", sourcefile, sourceline);
+	  va_start(argp, format);
+	  vfprintf(stderr, format, argp);
+	  va_end(argp);
+	  fprintf(stderr, "\n");
+	  if (use_errno && errno) perror("system error");
+	  fflush(stderr);
+	}  
+      else vsyslog(LOG_ERR, format, argp);
 
 #ifdef HAVE_MPI
       MPI_Initialized(&mpiflag);                 /* we're assuming we can do this, even in a corrupted, dying process...? */
@@ -300,18 +297,16 @@ esl_fatal(const char *format, ...)
   int mpiflag;
 #endif
   /* Check whether we are running as a daemon so we can do the right thing about logging instead of printing errors */
-  int parent_pid;
-  parent_pid = getppid();
-  if(parent_pid != 1){ // we're not running as a daemon, so print the error normally
-    va_start(argp, format);
-    vfprintf(stderr, format, argp);
-    va_end(argp);
-    fprintf(stderr, "\n");
-    fflush(stderr);
-  } 
-  else { // we are, so log the error instead of printing it
-       vsyslog(LOG_ERR, format, argp);
-  }
+  if (getppid() != 1)
+    { // we're not running as a daemon, so print the error normally
+      va_start(argp, format);
+      vfprintf(stderr, format, argp);
+      va_end(argp);
+      fprintf(stderr, "\n");
+      fflush(stderr);
+    } 
+  else vsyslog(LOG_ERR, format, argp);
+
 #ifdef HAVE_MPI
   MPI_Initialized(&mpiflag);
   if (mpiflag) MPI_Abort(MPI_COMM_WORLD, 1);
@@ -593,7 +588,7 @@ esl_dataheader(FILE *fp, ...)
       s   = va_arg(ap, char *);
       len = strlen(s);
       if (len > width) {
-	if (col == 0) ESL_XEXCEPTION(eslEINVAL, "esl_dataheader(): first arg (%s) too wide for %d-char column ('# ' leader took 2 chars)", col, s, width+2);
+	if (col == 0) ESL_XEXCEPTION(eslEINVAL, "esl_dataheader(): first arg (%s) too wide for %d-char column ('# ' leader took 2 chars)", s, width);
 	else          ESL_XEXCEPTION(eslEINVAL, "esl_dataheader(): arg %d (%s) too wide for %d-char column", col, s, width);
       }
 
@@ -637,6 +632,7 @@ esl_dataheader(FILE *fp, ...)
 /******************************************************************************
  * 4. Replacements for C library functions
  *  fgets()   ->  esl_fgets()     fgets() with dynamic allocation
+ *  printf()  ->  esl_printf()    printf() wrapped in our exception handling
  *  strdup()  ->  esl_strdup()    strdup() is not ANSI
  *  strcat()  ->  esl_strcat()    strcat() with dynamic allocation
  *  strtok()  ->  esl_strtok()    threadsafe strtok()
@@ -738,6 +734,77 @@ esl_fgets(char **buf, int *n, FILE *fp)
   return status;
 }
 
+
+/* Function:  esl_fprintf()
+ * Synopsis:  fprintf() wrapped in our exception handling
+ * Incept:    SRE, Thu Jun 14 09:43:59 2018 
+ *
+ * Purpose:   <fprintf()> wrapped in Easel exception handling. See
+ *            <esl_printf()> for rationale.
+ *
+ * Returns:   <eslOK> on success.
+ *
+ * Throws:    <eslEWRITE> on failure.
+ */
+int
+esl_fprintf(FILE *fp, const char *format, ...)
+{
+  if (fp && format)
+    {
+      va_list argp;
+      va_start(argp, format);
+      if ( vfprintf(fp, format, argp) < 0 ) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+      va_end(argp);
+    }
+  return eslOK;
+}
+
+
+
+/* Function:  esl_printf()
+ * Synopsis:  printf() wrapped in our exception handling
+ * Incept:    SRE, Thu Jun 14 09:17:17 2018 
+ *
+ * Purpose:   <printf()> wrapped in Easel exception handling. 
+ *
+ *            Rarely and insidiously, <printf()> can fail -- for
+ *            example, when output is redirected to a file and a disk
+ *            fills up. Every <printf()> needs to guard against this,
+ *            else output could silently fail. It seems slightly
+ *            cleaner to use Easel's idiomatic:
+ *
+ *            ```
+ *               if ((status = esl_printf(...)) != eslOK) return status; // no cleanup
+ *               if ((status = esl_printf(...)) != eslOK) goto ERROR;    // with cleanup
+ *            ```
+ *
+ *            as opposed to having to invoke
+ *            <ESL_EXCEPTION_SYS(eslEWRITE, "write failed")> each
+ *            time.
+ *
+ * Returns:   <eslOK> on success. 
+ *
+ * Throws:    <eslEWRITE> on failure.
+ */
+int
+esl_printf(const char *format, ...)
+{
+  if (format)
+    {
+      va_list argp;
+      va_start(argp, format);
+      if ( vprintf(format, argp) < 0 ) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+      va_end(argp);
+    }
+  return eslOK;
+}
+
+
+
+
+
+
+
 /* Function: esl_strdup()
  *
  * Purpose: Makes a duplicate of string <s>, puts it in <ret_dup>.
@@ -2000,7 +2067,7 @@ esl_tmpfile_named(char *basename6X, FILE **ret_fp)
 
   *ret_fp = NULL;
   old_mode = umask(077);
-  if ((fd = mkstemp(basename6X)) <  0)    return eslFAIL;
+  if ((fd = mkstemp(basename6X)) <  0)  return eslFAIL;
   umask(old_mode);
   if ((fp = fdopen(fd, "w+b")) == NULL) return eslFAIL;
 
@@ -2073,7 +2140,8 @@ esl_getcwd(char **ret_cwd)
  * 8. Typed comparison routines.
  *****************************************************************/
 
-/* Function:  esl_DCompare()
+/* Function:  esl_{DF}Compare()
+ * OBSOLETE. Use esl_{DF}CompareNew() instead.
  *
  * Purpose:   Compare two floating point scalars <a> and <b> for approximate equality.
  *            Return <eslOK> if equal, <eslFAIL> if not.
@@ -2114,6 +2182,7 @@ esl_FCompare(float a, float b, float tol)
 }
 
 /* Function:  esl_DCompareAbs()
+ * OBSOLETE. Use esl_{DF}CompareNew() instead.
  *
  * Purpose:   Compare two floating point scalars <a> and <b> for
  *            approximate equality, by absolute difference.  Return
@@ -2154,6 +2223,76 @@ esl_FCompareAbs(float a, float b, float tol)
 }
 
 
+/* Function:  esl_{DF}CompareNew()
+ * Synopsis:  Compare floating point values for approximate equality, better version.
+ * Incept:    SRE, Thu 19 Jul 2018 [Benasque]
+ *
+ * Purpose:   Return <eslOK> if <x0> and <x> are approximately equal within
+ *            relative tolerance tolerance <rtol> and absolute
+ *            tolerance <atol>;  <eslFAIL> if not.
+ *            
+ *            Equality is defined as $|x0-x| < |x0|*rtol + atol$.
+ *            
+ *            <x0> is the reference value: the true value or the
+ *            better estimate, if appropriate. For example, if you are
+ *            comparing a new (better) estimate $x_i$ to a previous
+ *            (worse) estimate $x_{i-1}$, <x0> is the new, <x> is the
+ *            old.
+ *
+ *            Tolerances <rtol> and <atol> must be $\geq 0$. For a
+ *            strictly relative tolerance test, use <atol=0>; for
+ *            strict absolute tolerance, use <rtol=0>.
+ *            
+ *            "Approximate equality" in floating point math: here be
+ *            dragons. Usually you want to compare floating point
+ *            values by their relative difference <rtol>. An <rtol> of
+ *            $1e-5$ essentially means they agree up to their first
+ *            five digits, regardless of absolute magnitude. However,
+ *            relative difference fails for <x0 ~ 0>.  Using both
+ *            <rtol> and <atol>, there is a switch at <|x0| = atol /
+ *            rtol>: above this |x0|, <rtol> dominates, and below it,
+ *            <atol> does. So you typically want <atol << rtol>, so
+ *            the switchover only happens close to zero.
+ *            
+ *            In floating point math, the smallest possible |x0-x| is
+ *            on the order of |x0| times machine epsilon, where
+ *            typically <DBL_EPSILON> is 2.2e-16; <FLT_EPSILON>,
+ *            1.2e-7, so it does not make sense to set <atol> smaller
+ *            than this.
+ *            
+ *            Special values: comparisons involving <NaN> and <inf>
+ *            are always <eslFAIL>; <inf> because difference
+ *            <inf-inf=inf>, and <NaN> because IEEE754. 
+ *            
+ *            Note that <eslOK> has value 0, not TRUE, so you don't want to
+ *            write code like <if (esl_DCompare()) ...>; you want to
+ *            test explicitly against <eslOK>.
+ *
+ * Args:      x0   - reference value to compare against (either true, or better estimate)
+ *            x    - test value 
+ *            rtol - relative tolerance
+ *            atol - absolute tolerance
+ *
+ * Returns:   <eslOK> if <x0> and <x> are approximately equal.
+ *            <eslFAIL> if not.
+ *
+ * Xref:      H5/116
+ *
+ * Note:      <, not <=, because with <=, -inf ~= inf (yes really).
+ */
+int
+esl_DCompareNew(double x0, double x, double rtol, double atol)
+{
+  if (fabs(x0 - x) < rtol * fabs(x0) + atol) return eslOK;
+  return eslFAIL;
+}
+int
+esl_FCompareNew(float x0, float x, float rtol, float atol)
+{
+  if (fabs(x0 - x) < rtol * fabs(x0) + atol)  return eslOK;
+  return eslFAIL;
+}
+
 
 
 
@@ -2304,7 +2443,6 @@ utest_strtok(void)
   if (*s != '\0')                                                    esl_fatal(msg);
 
   free(teststring);
-  return;
 }
   
 static void
@@ -2353,7 +2491,6 @@ utest_FileExists(void)
 
   remove(tmpfile);
   if (esl_FileExists(tmpfile))   esl_fatal(msg);
-  return;
 }
 
 static void
@@ -2372,9 +2509,36 @@ utest_tmpfile_named(void)
   if (strcmp(buf, "Unit test.\n")  != 0)     esl_fatal(msg);
   fclose(fp);
   remove(tmpfile);
-  return;
 }
 
+static void
+utest_compares(void)
+{
+  char msg[] = "easel utest_compares failed";
+
+  // if (esl_DCompare(-eslINFINITY, eslINFINITY, 1e-5) != eslFAIL) esl_fatal(msg);   /* -inf != inf */
+  // if (esl_DCompare(eslNaN, eslNaN, 1e-5) != eslFAIL) esl_fatal(msg);              /* NaN fails in any comparison */
+  if (esl_DCompare(0.,           eslNaN,        1e-12) != eslFAIL) esl_fatal(msg);
+  if (esl_DCompare(eslNaN,       0.,            1e-12) != eslFAIL) esl_fatal(msg);
+  //  if (esl_DCompare(eslINFINITY,  eslINFINITY,   1e-12) != eslFAIL) esl_fatal(msg);  
+
+  if (esl_DCompareNew(-eslINFINITY, eslINFINITY,   1e-12, 1e-16) != eslFAIL) esl_fatal(msg);   /* -inf != inf of course*/
+  if (esl_DCompareNew(eslINFINITY,  eslINFINITY,   1e-12, 1e-16) != eslFAIL) esl_fatal(msg);   /* inf != inf too, because rel diff = inf */
+  if (esl_DCompareNew(eslNaN,       eslNaN,        1e-12, 1e-16) != eslFAIL) esl_fatal(msg);   /* NaN fails in any comparison */
+  if (esl_DCompareNew(0.,           eslNaN,        1e-12, 1e-16) != eslFAIL) esl_fatal(msg);   
+  if (esl_DCompareNew(eslNaN,       0.,            1e-12, 1e-16) != eslFAIL) esl_fatal(msg);
+  if (esl_DCompareNew(0.,           1e-17,         1e-12, 1e-16) != eslOK)   esl_fatal(msg);
+
+  if (esl_FCompareNew(-eslINFINITY, eslINFINITY,   1e-6, 1e-10) != eslFAIL) esl_fatal(msg);   /* -inf != inf of course*/
+  if (esl_FCompareNew(eslINFINITY,  eslINFINITY,   1e-6, 1e-10) != eslFAIL) esl_fatal(msg);   /* inf != inf too, because rel diff = inf */
+  if (esl_FCompareNew(eslNaN,       eslNaN,        1e-6, 1e-10) != eslFAIL) esl_fatal(msg);   /* NaN fails in any comparison */
+  if (esl_FCompareNew(0.,           eslNaN,        1e-6, 1e-10) != eslFAIL) esl_fatal(msg);   
+  if (esl_FCompareNew(eslNaN,       0.,            1e-6, 1e-10) != eslFAIL) esl_fatal(msg);
+  if (esl_DCompareNew(0.,           1e-11,         1e-6, 1e-10) != eslOK)   esl_fatal(msg);
+}
+
+
+
 #endif /*eslEASEL_TESTDRIVE*/
 
 
@@ -2401,6 +2565,8 @@ int main(void)
   utest_sprintf();
   utest_FileExists();
   utest_tmpfile_named();
+  utest_compares();
+  
   return eslOK;
 }
 #endif /*eslEASEL_TESTDRIVE*/
