@@ -122,9 +122,19 @@ sub calculateF {
     if ($th+$fh > 0.0) { $F = 2.0 * $tph / ($th + $fh) }
     else               { $F = 0.0 }
 
-    $$ret_sen = 100.*$sen;
-    $$ret_ppv = 100.*$ppv;
-    $$ret_F   = 100.*$F;
+    $$ret_sen = int(10000*$sen)/100;
+    $$ret_ppv = int(10000*$ppv)/100;
+    $$ret_F   = int(10000*$F)/100;
+}
+sub calculateSEN {
+    my ($tph, $th, $ret_sen) = @_;
+    
+    my $sen;
+
+    if ($tph > $th) { print "trues found $tph > trues $th!!\n"; die; }
+    $sen = ($th > 0)? $tph/$th : 0.0;
+
+    $$ret_sen = int(10000*$sen)/100;
 }
 
 sub dump_histogram {
@@ -149,7 +159,8 @@ sub fill_histo_array {
     if ($sc <=  -$shift)   { $his_ref->[0]    += $val; return; }
     
     for (my $i=0; $i<=$dim; $i++) { 
-	if ( $i/$k-$shift <= $sc && $sc < ($i+1)/$k - $shift) {  
+	if ( $i/$k-$shift <= $sc && $sc < ($i+1)/$k - $shift) {
+	    #print "sc $sc add at $i\n";
 	    $his_ref->[$i] += $val;
 	    last; 
 	} 
@@ -219,17 +230,17 @@ sub gnuplot_histo {
     print GP "set xrange [$xleft:$xright]\n";
     if ($ymax > 0) { print GP "set yrange [0:$ymax]\n"; }
 
-    #print GP "set title \"$title\\n\\n$key\"\n";
-    print GP "set title '$title'\n";
+    print GP "set title \"$title\\n\\n$key\"\n";
+    #print GP "set title '$title'\n";
 
     print GP "set ylabel '$ylabel'\n";
 
     my $cmd = "";
     if ($iscum) {
-	$cmd .= "'$hfile' using $xfield:$yfield  with lines title '$key' ls 1";
+	$cmd .= "'$hfile' using $xfield:$yfield  with lines title '' ls 1";
     }
     else {
-	$cmd .= "'$hfile' using $xfield:$yfield  with boxes title '$key' ls 1";
+	$cmd .= "'$hfile' using $xfield:$yfield  with boxes title '' ls 1";
     } 
 
     #print  "plot $cmd\n";
@@ -373,13 +384,13 @@ sub gnuplot_ave_histo_with_dots {
 sub gnuplot_define_styles {
     my ($gp) = @_;
     print $gp "set style line 1   lt 1 lc rgb 'black'   pt 1 ps 0.5 lw 1\n";
-    print $gp "set style line 2   lt 1 lc rgb 'brown'   pt 1 ps 0.5 lw 1\n";
+    print $gp "set style line 2   lt 1 lc rgb 'brown'   pt 1 ps 0.6 lw 1\n";
     print $gp "set style line 3   lt 1 lc rgb 'grey'    pt 1 ps 0.5 lw 1\n";
     print $gp "set style line 4   lt 1 lc rgb 'cyan'    pt 1 ps 0.5 lw 3\n";
     print $gp "set style line 7   lt 1 lc rgb 'red'     pt 1 ps 0.5 lw 3\n";
     print $gp "set style line 5   lt 1 lc rgb 'purple'  pt 1 ps 0.5 lw 3\n";
     print $gp "set style line 6   lt 1 lc rgb 'orange'  pt 1 ps 0.5 lw 3\n";
-    print $gp "set style line 8   lt 1 lc rgb 'blue'    pt 1 ps 0.5 lw 3\n";
+    print $gp "set style line 8   lt 1 lc rgb 'blue'    pt 1 ps 0.6 lw 3\n";
     print $gp "set style line 9   lt 2 lc rgb 'magenta' pt 1 ps 0.5 lw 3\n";
    
     print $gp "set style line 88   lt 1 lc rgb 'cyan'   pt 7 pi -1  ps 1.0 lw 2\nset pointintervalbox 1\n";
@@ -389,9 +400,9 @@ sub gnuplot_define_styles {
     print $gp "set style line 1111   lt 1 lc rgb 'black'      pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
     print $gp "set style line 1113   lt 1 lc rgb 'brown'      pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n"; 
     print $gp "set style line 1112   lt 1 lc rgb 'blue'       pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
-    print $gp "set style line 1116   lt 1 lc rgb '#8A2BE2'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
-    print $gp "set style line 1114   lt 1 lc rgb '#D3D3D3'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n"; #light gray
-    print $gp "set style line 1115   lt 1 lc rgb '#FF8C00'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
+    print $gp "set style line 1115   lt 1 lc rgb '#8A2BE2'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
+    print $gp "set style line 1116   lt 1 lc rgb '#D3D3D3'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n"; #light gray
+    print $gp "set style line 1114   lt 1 lc rgb '#FF8C00'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
     print $gp "set style line 1117   lt 1 lc rgb 'red'        pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
     print $gp "set style line 1118   lt 1 lc rgb '#8B4513'    pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n"; #saddle brown
     print $gp "set style line 1119   lt 1 lc rgb 'blueviolet' pt 31 pi -1  lw 2 ps 0.5 \nset pointintervalbox 1\n";
@@ -1010,9 +1021,9 @@ sub  hit2trace {
 	    ${$tr_ref->[$ntr]->{"TRACE::i"}}[$N] = $i;
 
 	    if    (!FUNCS::isgap($qchar) && !FUNCS::isgap($tchar)) { $k += $addk; $i += $addi; $st = 0; $tsq .= uc($tchar); } # match
-	    elsif (!FUNCS::isgap($qchar))                   { $k += $addk;              $st = 1;                     } # deletion
-	    elsif (!FUNCS::isgap($tchar))                   {              $i += $addi; $st = 2; $tsq .= lc($tchar); } # insertion
-	    else                                     { print "cannot have two gaps\n";                   die; }
+	    elsif (!FUNCS::isgap($qchar))                          { $k += $addk;              $st = 1;                     } # deletion
+	    elsif (!FUNCS::isgap($tchar))                          {              $i += $addi; $st = 2; $tsq .= lc($tchar); } # insertion
+	    else                                                   { print "cannot have two gaps\n";                   die; }
 
 	    ${$tr_ref->[$ntr]->{"TRACE::k"}}[$N]  = $k;
 	    ${$tr_ref->[$ntr]->{"TRACE::st"}}[$N] = $st;
@@ -1022,8 +1033,7 @@ sub  hit2trace {
 	$tr_ref->[$ntr]->{"TRACE::N"}  = $N;
 	$tr_ref->[$ntr]->{"TRACE::sq"} = $tsq;
 
-	if (1||$verbose) {
-	    #printtrace($tr_ref->[$ntr]);
+	if ($verbose) {
 	    printhit($hit_ref->[$h]);
 	}
 
@@ -1141,7 +1151,7 @@ sub make_msa {
 	my $N    = $tr_ref->[$t]->{"TRACE::N"};
 	my $sq   = $tr_ref->[$t]->{"TRACE::sq"};
 	my $ib   = ${$tr_ref->[$t]->{"TRACE::i"}}[0];
- 	my $ie   = ${$tr_ref->[$t]->{"TRACE::i"}}[$N-1];
+ 	my $ie   = ($N>0)? ${$tr_ref->[$t]->{"TRACE::i"}}[$N-1] : $ib-1;
  	my $apos = 0;
 
 	for (my $z = 0; $z < $N ; $z++) 
@@ -1281,6 +1291,7 @@ sub merge_aseqs {
 	    my $coords1;
 	    get_name($asqname_ref->[$s1], \$name1, \$coords1);
 	    
+	    if($name=~/60_9HOMO/&&$name1=~/60_9HOMO/) { printf("$name\n$asq\n$asq1 aredist? %d\n", are_disjoint($asq, $asq1)); }
 	    if ($name1 =~ /^$name$/ && are_disjoint($asq, $asq1)) {
 		printf "  .... MERGE ($s1) %s with ($s) %s new_NASQ = %d\n", $asqname_ref->[$s1], $asqname_ref->[$s], $nasq-1;
 		$asqname_ref->[$s] = "$name/$coords//$coords1";
@@ -1693,6 +1704,8 @@ sub parse_hmmout {
 
     my $hashits = 0;
     my $tname;
+
+    my $bit = 0;
     
     print "\nquery $whichQ: getting hits from file: $file\n";
     open (FILE, "$file") || die;
@@ -1721,21 +1734,14 @@ sub parse_hmmout {
 	    $tname = "$targetname\/$loci";
 	    if ($NHIT > 0 && $nhit+1 >= $NHIT) { last; }
 	}
-	elsif($thisquery && $annote && 
-	      /(\S+)\s+\S+\s+(\S+)\s+\d+\s+\d+\s[\.\[][\.\]]\s+\d+\s+\d+\s[\.\[][\.\]]\s+\d+\s+\d+\s[\.\[][\.\]]\s+(\d+)\s+\S+\s*$/) {
-	    $sc        = $1;
-	    $Eval      = $2;
-	    $targetlen = $3;
-	    $complete_hit = 0;
-
-	    if ($nhit >= 0) {
-		if ( abs($hit[$nhit]->{"HIT::qj"}-$hit[$nhit]->{"HIT::qi"})+1 == $querylen) { $complete_hit = 1; }
-	    }
-	    if ($complete_hit) { last; }
+  	elsif($thisquery && $annote && 
+	      /==\s+domain\s+\d+\s+score:\s+(\S+)\s+bits\;\s+conditional E-value:\s+(\S+)\s*/) {
+	    $sc    = $1;
+	    $Eval  = $2;
 	    
 	    if (takehit($targetname, $loci, $nsp, $species_ref, $loci_ref) && ($targetE < 0 || $Eval <= $targetE)) {
 		$nhit ++;
-		print "...new nhit $tname Eval $Eval\n";
+		print "...new nhit $nhit $tname Eval $Eval\n";
 		if ($Eval < $min_eval) { $min_eval = $Eval; }
 		if ($Eval > $max_eval) { $max_eval = $Eval; }
 		
@@ -1755,29 +1761,41 @@ sub parse_hmmout {
 	    }
 	    else { $thisquery = 0; }
 	}
-	elsif($thisquery && /^\s*$queryname\s+(\d+)\s+(\S+)\s+(\d+)\s*$/) {
+	elsif($thisquery && $bit == 0 && /^\s*$queryname\s+(\d+)\s+(\S+)\s+(\d+)\s*$/) {
 	    my $i = $1;
 	    my $j = $3;
+	    $bit = 1;
+	    if ($nhit < 0) { print " you should have found a hit here already\n"; die; }
+	    
 	    if ($hit[$nhit]->{"HIT::qi"} == 0) { $hit[$nhit]->{"HIT::qi"} = $i; }
 	    $hit[$nhit]->{"HIT::qj"} = $j; 
 
+	    if ($i == 0 || $j == 0) { printf("bad query bounds for hit $nhit [$i,$j] targetname $targetname\n"); die; }
+	    
 	    $hit[$nhit]->{"HIT::qasq"} .= $2;
 	}
 	elsif($thisquery && /\s+(\d+)\s+(\S+)\s+(\d+)\s*$/) {
 	    my $i = $1;
 	    my $j = $3;
+	    if ($nhit < 0) { print " you should have found a hit here already\n"; die; }
+	    
 	    if ($hit[$nhit]->{"HIT::ti"} == 0) { $hit[$nhit]->{"HIT::ti"} = $i; }
-	    $hit[$nhit]->{"HIT::tj"} = $j; 
-						 
+	    $hit[$nhit]->{"HIT::tj"} = $j;
+	    if ($nhit == 2316) {
+		printf("ti $i tj $j\n");
+	    }
+
+	    if ($i == 0 || $j == 0) { printf("bad target bound for hit $nhit [$i,$j] targetname $targetname\n"); die; }
+
 	    $hit[$nhit]->{"HIT::tasq"} .= $2;
+	    $bit = 0;
 	}
     }
     close (FILE);
     $nhit ++;
 
-    print "..nhits $nhit\n";
-
     trimhits_by_query (\$nhit, \@hit, $hmmfrom, $hmmto);
+    
     trimhits_by_target(\$nhit, \@hit, $target, $targetfrom, $targetto);
     hit2trace($nhit, \@hit, $ret_ntr, $tr_ref, $verbose);
 
@@ -1898,33 +1916,37 @@ sub parse_sqfile {
 }
 
 sub parse_stofile {
-    my ($sqfile, $ret_nsq, $name_ref, $sq_ref, $ret_ss, $ct_ref) = @_;
+    my ($sqfile, $ret_nsq, $name_ref, $sq_ref, $ret_ss, $ct_ref, $ret_rfsq) = @_;
 
     my $nsq = 0;
     my $sq = "";
     my $ss = "";
     my $name = "";
+    my $rfsq = "";
 
     my $b = 0;
     open(FILE, "$sqfile") || die;
     while (<FILE>) {
 	if (/# STO/) {
 	}
-	elsif (/#=GC SS_cons\s+(\S+)\s*$/) {
+	elsif (/#=GC\s+SS_cons\s+(\S+)\s*$/) {
 	    $ss .= "$1";
 	}
+	elsif (/#=GC\s+RF\s+(\S+)\s*$/) {
+	    $rfsq .= "$1";
+	}
 	elsif (/^#/) {
 	}
 	elsif (/^$/) {
 	    $b ++;
 	    $nsq = 0;
 	}
-	elsif ($b < 2 && /^(\S+)\s+(\S+)\s*$/) {
+	elsif (/^([^#]\S+)\s+(\S+)\s*$/) {
 	    $name_ref->[$nsq]  = $1;
 	    $sq_ref->[$nsq]   .= "$2";
 	    $nsq ++;
 	}
-	elsif ($b > 1 && /^(\S+)\s+(\S+)\s*$/) {
+	elsif ($nsq > 0 && /^(\S+)\s+(\S+)\s*$/) {
 	    my $name  = $1;
 	    my $sq    = $2;
 	    
@@ -1932,15 +1954,16 @@ sub parse_stofile {
 		$sq_ref->[$nsq] .= "$sq";
 		$nsq ++;
 	    }
-	    else { print "bad sq\n"; die; }
+	    else { print "bad sq $name | $name_ref->[$nsq]\n"; die; }
 	}
     }
     close(FILE);
 
     ss2ct($ss, $ct_ref);
 
-    $$ret_ss = $ss;
-    $$ret_nsq = $nsq;
+    $$ret_ss   = $ss;
+    $$ret_rfsq = $rfsq;
+    $$ret_nsq  = $nsq;
  
     return length($ss);
 }
@@ -1949,6 +1972,8 @@ sub ss2ct {
     my ($ss, $ct_ref) = @_;
 
     my $len = length($ss);
+    if ($len <= 0) { return; }
+    
     my @pda;
     my @pdapk;
     my $pair;
@@ -2386,8 +2411,8 @@ sub plot_id2bench {
 sub  printhit {
     my ($hit) = @_;
     
-    printf "query:  %d-%d\n%s\n", $hit->{"HIT::qi"}, $hit->{"HIT::qj"}, $hit->{"HIT::qasq"};
-    printf "target: %d-%d\n%s\n", $hit->{"HIT::ti"}, $hit->{"HIT::tj"}, $hit->{"HIT::tasq"};
+    printf "query %s:  %d-%d\n%s\n", $hit->{"HIT::qname"}, $hit->{"HIT::qi"}, $hit->{"HIT::qj"}, $hit->{"HIT::qasq"};
+    printf "target %s: %d-%d\n%s\n", $hit->{"HIT::tname"}, $hit->{"HIT::ti"}, $hit->{"HIT::tj"}, $hit->{"HIT::tasq"};
 }
 
 sub  printtrace {
@@ -2520,8 +2545,9 @@ sub sorted_files {
     my ($dir, $files_ref, $suffix, $prefix) = @_;
     local *DIRH;
     opendir DIRH, $dir or die "eh? $dir: $!";
+    
     if ($prefix) {
-	@$files_ref = grep ( /$prefix\S+\.$suffix$/, map { "$dir/$_" } readdir(DIRH));
+	@$files_ref = grep ( /\S*$prefix\S+\.$suffix$/, map { "$dir/$_" } readdir(DIRH));
     }
     else {
 	@$files_ref = grep ( /^\S+\.$suffix$/, map { "$dir/$_" } readdir(DIRH));
@@ -2611,7 +2637,7 @@ sub  trimhits_by_query {
 	$hit_ref->[$h]->{"HIT::qj"} = $newqj;
 	$hit_ref->[$h]->{"HIT::ti"} = $newti;
 	$hit_ref->[$h]->{"HIT::tj"} = $newtj;
-
+	
 	$hit_ref->[$h]->{"HIT::qasq"} = $newqasq;
  	$hit_ref->[$h]->{"HIT::tasq"} = $newtasq;
 	
@@ -2752,8 +2778,8 @@ sub write_ave_histogram {
     my $ave;
     my $std;
     histo_stats($N, $k, $shift, $histo_ref, \$median, \$ave, \$std);
-    if ($verbose) { printf "ave = %.4f +/- %.4f MEDIAN = %f file %s\n", $ave, $std, $median, $hfile; }
-    if ($his) { printf $his "#ave = %.4f +/- %.4f MEDIAN = %.4f\n", $ave, $std, $median; }
+    if ($verbose) { printf      " ave = %.4f +/- %.4f MEDIAN = %f file %s\n", $ave, $std, $median, $hfile; }
+    if ($his)     { printf $his "#ave = %.4f +/- %.4f MEDIAN = %.4f\n", $ave, $std, $median; }
 
     for (my $i=0; $i<=$dim; $i++) { 
 	my $len = $i/$k + 0.5/$k - $shift;
