@@ -39,10 +39,9 @@ static int    is_wc(int x, int y);
 static int    is_allowed_pair(int x, int y, ESL_DMATRIX *allowpair);
 static int    number_pairs(int L, int *ct);
 static int    mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s *mi,
-				int donull2b, double tol, int verbose, char *errbuf);
+				double tol, int verbose, char *errbuf);
 static int    mutual_postorder_ppij(int i, int j, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ribosum, struct mutual_s *mi,
 				    ESL_DMATRIX **CL, ESL_DMATRIX **CR, double tol, int verbose, char *errbuf);
-static int    shuffle_null2b_col(ESL_RANDOMNESS *r, ESL_ALPHABET *abc, int nseq, int *col, int *paircol, int **ret_shcol, char *errbuf);
 static int    shuffle_col(ESL_RANDOMNESS *r, int nseq, int *useme, int *col, int **ret_shcol, char *errbuf);
 
 int                 
@@ -322,7 +321,7 @@ corr_CalculateGT(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
   int              verbose = data->verbose;
   int              i, j;
   int              status = eslOK;
- 
+
   switch (covclass) {
   case C16:
     status = corr_CalculateGT_C16 (mi, verbose, errbuf);
@@ -335,24 +334,23 @@ corr_CalculateGT(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
       status = corr_CalculateGT_C2(mi, data->allowpair, verbose, errbuf);
     }
     else {
-      status = corr_CalculateGT_C16(mi, verbose, errbuf);
+     status = corr_CalculateGT_C16(mi, verbose, errbuf);
     }
     break;
   }
   
   if (verbose) {
-    printf("GTrange[%f,%f] clase %d\n", mi->minCOV, mi->maxCOV, covclass);
     for (i = 0; i < mi->alen-1; i++) 
       for (j = i+1; j < mi->alen; j++) {
 	if (data->msamap[i]==15&&data->msamap[j]==70) printf("GT[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
       } 
   }
-  
+
   if (analyze) {
     status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
     if (status != eslOK) goto ERROR;
   }
-  
+ 
   return status;
   
  ERROR:
@@ -407,6 +405,7 @@ corr_CalculateGT_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, ch
   double exp_wc, exp_nwc;
   int    i, j;
   int    x, y;
+  
 #if GAPASCHAR
   int    K = mi->abc->K+1;
 #else
@@ -879,7 +878,7 @@ corr_CalculateRAF(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int anal
       }
       qij /= msa->nseq;
       
-      cij /= (msa->nseq > 1)? (double)msa->nseq * ((double)msa->nseq-1.0): 1.0;
+      cij /= (msa->nseq > 1)? (double)msa->nseq * ((double)msa->nseq-1.0) : 1.0;
       cij *= 2.0;
       mi->COV->mx[i][j] = mi->COV->mx[j][i] = cij - psi * qij;
       if (mi->COV->mx[i][j] < mi->minCOV) mi->minCOV = mi->COV->mx[i][j];
@@ -891,7 +890,7 @@ corr_CalculateRAF(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int anal
     printf("RAF[%f,%f]\n", mi->minCOV, mi->maxCOV);
     for (i = 0; i < mi->alen-1; i++) 
       for (j = i+1; j < mi->alen; j++) {
-	if (i==5&&j==118) printf("RAF[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
+	printf("RAF[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
       } 
   }
   
@@ -1055,7 +1054,7 @@ corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANK
   int              L = mi->alen;
   int              i, j;
   int              status = eslOK;
-  
+
   corr_COVTYPEString(&type, mi->type, errbuf);
 
   switch(actype) {
@@ -1122,9 +1121,10 @@ corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANK
       } 
   }
 
-  if (analyze) 
+  if (analyze) {
     status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-  if (status != eslOK) goto ERROR;
+    if (status != eslOK) goto ERROR;
+  }
   
   free(type);
   free(covtype);
@@ -1272,7 +1272,7 @@ corr_Destroy(struct mutual_s *mi)
 
 
 int 
-corr_NaivePP(ESL_RANDOMNESS *r, ESL_MSA *msa, struct mutual_s *mi, int donull2b, double tol, int verbose, char *errbuf)
+corr_NaivePP(ESL_RANDOMNESS *r, ESL_MSA *msa, struct mutual_s *mi, double tol, int verbose, char *errbuf)
 {
   int64_t alen = msa->alen;
   int     i, j;
@@ -1280,7 +1280,7 @@ corr_NaivePP(ESL_RANDOMNESS *r, ESL_MSA *msa, struct mutual_s *mi, int donull2b,
 
   for (i = 0; i < alen-1; i ++)
     for (j = i+1; j < alen; j ++) {
-      status = mutual_naive_ppij(r, i, j, msa, mi, donull2b, tol, verbose, errbuf);
+      status = mutual_naive_ppij(r, i, j, msa, mi, tol, verbose, errbuf);
       if (status != eslOK) goto ERROR;
     }
   
@@ -1376,7 +1376,7 @@ corr_PostOrderPP(ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ribosum, struct
 
 int                 
 corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ribosum, struct mutual_s *mi, 
-	   METHOD method, int donull2b, double tol, int verbose, char *errbuf)
+	   METHOD method, double tol, int verbose, char *errbuf)
 {
   int i, j;
   int x, y;
@@ -1390,7 +1390,7 @@ corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ri
   switch(method) {
   case NONPARAM:
   case POTTS:
-    status = corr_NaivePP(r, msa, mi, donull2b, tol, verbose, errbuf);
+    status = corr_NaivePP(r, msa, mi, tol, verbose, errbuf);
     if (status != eslOK) goto ERROR;    
     break;
   case AKMAEV:
@@ -1636,10 +1636,10 @@ number_pairs(int L, int *ct)
 
 static int    
 mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s *mi,
-		  int donull2b, double tol, int verbose, char *errbuf)
+		  double tol, int verbose, char *errbuf)
 {
-  int    *coli = NULL;
-  int    *colj = NULL;
+  int    *coli   = NULL;
+  int    *colj   = NULL;
   int    *shcoli = NULL;
   int    *shcolj = NULL;
 #if GAPASCHAR
@@ -1656,24 +1656,17 @@ mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s
   esl_vec_DSet(mi->pp[i][j], K2, 1e-5); //some prior to avoid zeros
   mi->nseff[i][j] = 0.;
 
-  ESL_ALLOC(coli, sizeof(int)*msa->nseq);
-  ESL_ALLOC(colj, sizeof(int)*msa->nseq);
+  ESL_ALLOC(coli, sizeof(int) * msa->nseq);
+  ESL_ALLOC(colj, sizeof(int) * msa->nseq);
   for (s = 0; s < msa->nseq; s ++) {
     coli[s] = msa->ax[s][i+1];
     colj[s] = msa->ax[s][j+1];
   }
 
   /* shuffle in each column residues that appear to be canonical (i,j) pairs */
-  if (donull2b) {
-    status = shuffle_null2b_col(r, msa->abc, msa->nseq, coli, colj, &shcoli, errbuf);
-    if (status != eslOK) goto ERROR;
-    status = shuffle_null2b_col(r, msa->abc, msa->nseq, colj, coli, &shcolj, errbuf);
-    if (status != eslOK) goto ERROR;
-  }
-
   for (s = 0; s < msa->nseq; s ++) {
-    resi = (donull2b)? shcoli[s] : coli[s];
-    resj = (donull2b)? shcolj[s] : colj[s];
+    resi = coli[s];
+    resj = colj[s];
     
     if (esl_abc_XIsCanonical(msa->abc, resi) && esl_abc_XIsCanonical(msa->abc, resj)) { 
       mi->nseff[i][j]                += msa->wgt[s];
@@ -1867,36 +1860,6 @@ mutual_postorder_ppij(int i, int j, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix
   return status;
 }
 
-int 
-shuffle_null2b_col(ESL_RANDOMNESS *r, ESL_ALPHABET *abc, int nseq, int *col, int *paircol, int **ret_shcol, char *errbuf)
-{
-  int *useme = NULL;
-  int *shcol = NULL;
-  int  s;
-  int  status;
-  
-  /* allocation */
-  ESL_ALLOC(useme, sizeof(int) * nseq); // vecto to mark residues in the column
- 
-  /* shuffle only positions with residues and with canonical pair in the other column */
-  esl_vec_ISet(useme, nseq, FALSE);
-  for (s = 0; s < nseq; s ++) 
-    if ( esl_abc_XIsResidue(abc,col[s]) && is_wc(col[s], paircol[s]) ) useme[s] = TRUE;
- 
-  /* within colum permutation */
-  status = shuffle_col(r, nseq, useme, col, &shcol, errbuf);
-  if (status != eslOK) goto ERROR;
-
-  *ret_shcol = shcol;
-
-  free(useme);
-  return eslOK;
-  
- ERROR:
-  if (shcol)  free(shcol);
-  if (useme)  free(useme);
-  return status;
-}
 
 int 
 shuffle_col(ESL_RANDOMNESS *r, int nseq, int *useme, int *col, int **ret_shcol, char *errbuf)
