@@ -385,7 +385,7 @@ msamanip_RemoveGapColumns(double gapthresh, ESL_MSA *msa, int64_t startpos, int6
       ESL_XFAIL(eslFAIL, errbuf, "RemoveGapColumns(): error in esl_msa_ColumnSubset");
   }  
 
-  if (msa->alen == 0) ESL_XFAIL(eslFAIL, errbuf, "no positions left after grap trimming");
+  if (msa->alen == 0) ESL_XFAIL(eslFAIL, errbuf, "no positions left after gap trimming");
   ESL_ALLOC(map, sizeof(int) * msa->alen);
   for (apos = 0; apos < alen; apos++) 
     if (useme[apos]) map[newpos++] = apos + startpos;
@@ -424,7 +424,7 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
   ESL_DSQ *dsq = NULL;
   int     *useme = NULL;
   double   flen;
-  int64_t  clen = 0;           // seq_cons length 
+  double   clen = 0.;           // seq_cons length, otherwise avg lenght of sequences
   int64_t  alen;
   int64_t  len;
   int      n;
@@ -435,28 +435,32 @@ msamanip_RemoveFragments(float fragfrac, ESL_MSA **msa, int *ret_nfrags, int *re
   for (n = 0; n < omsa->ngc; n++) {
     if (strcmp(omsa->gc_tag[n], "seq_cons") == 0) {
       esl_abc_CreateDsq(omsa->abc, omsa->gc[n], &dsq);
-      clen = esl_abc_dsqrlen(omsa->abc, dsq);
+      clen = (double)esl_abc_dsqrlen(omsa->abc, dsq);
       alen = esl_abc_dsqlen(dsq);
       break;
     }
   }
-  
+
+  // if consensus length is not given, calculate fragments relative to
+  //  the average length of all the sequences
   ESL_ALLOC(useme, sizeof(int) * omsa->nseq);
-  if (clen == 0) { /* calculate fragments relative to the largest sequence */
+  if (clen == 0) { 
     for (i = 0; i < omsa->nseq; i++)  {
       useme[i] = 0;
       for (x = 1; x < omsa->alen; x++) { 
 	if (esl_abc_XIsResidue(omsa->abc, omsa->ax[i][x])) useme[i] ++;
       }
-      if (useme[i] > clen) clen = useme[i];
+      clen += (double)useme[i];
     }
-    flen = (double)clen * fragfrac;
+    clen /= (omsa->nseq > 0)? (double)omsa->nseq : 0;
+    
+    flen = clen * fragfrac;
     for (i = 0; i < omsa->nseq; i++)  {
        useme[i] = (useme[i] <= flen)? FALSE : TRUE;
     }
   } 
   else { /* for each seq in msa, calculate number of residues that match the seq_cons */
-    flen = (double)clen * fragfrac;
+    flen = clen * fragfrac;
     
     for (i = 0; i < omsa->nseq; i++)  {
       len = 0;
@@ -511,7 +515,7 @@ msamanip_Truncate(ESL_MSA *msa, int64_t tstart, int64_t tend, int64_t *ret_start
    */
   from = tstart-1;
   to   = tend-1;
-  if (from >= to) ESL_XFAIL(eslFAIL, errbuf, "Truncation error from %d >= to %d\n", from+1, to+1);
+  if (from >= to) ESL_XFAIL(eslFAIL, errbuf, "Truncation error from %d > to %d\n", from+1, to+1);
 
   /* create the truncation mask */
   ESL_ALLOC(useme, sizeof(int) * msa->alen);
@@ -737,7 +741,7 @@ msamanip_SelectSubset(ESL_RANDOMNESS  *r, int nseq, ESL_MSA **omsa, char **msafi
   if (nseq == 0) {  
     esl_msa_Destroy(msa); *omsa = NULL; return eslOK; 
   }
-  if (nseq >= msa->nseq) return eslOK; 
+  if (nseq > msa->nseq) ESL_XFAIL(eslFAIL, errbuf, "cannot sample %d sequences. Alignment has %d sequences.", nseq, msa->nseq);
   
 
   /* the newfile file with submsa */
@@ -1209,7 +1213,7 @@ msamanip_ShuffleWithinRow(ESL_RANDOMNESS  *r, ESL_MSA *msa, ESL_MSA **ret_shmsa,
 	for (i = 0; i < msa->alen; i++) if (useme[i] == TRUE) { colidx[c] = i; c++; }
 	ESL_ALLOC(perm,  sizeof(int) * alen);
 	
-	for (c = 0; c < alen; i ++) perm[c] = c;
+	for (c = 0; c < alen; c ++) perm[c] = c;
  	if ((status = esl_vec_IShuffle(r, perm, alen)) != eslOK) ESL_XFAIL(status, errbuf, "failed to randomize perm");
 
 	c = 0;
@@ -1365,7 +1369,6 @@ esl_msaweight_Gremlin(ESL_MSA *msa, double reweight_thresh, int isplmDCA, char *
   int          nsq = msa->nseq;
   int          L   = msa->alen;
   int          K   = msa->abc->K;
-  int          Kg  = K+1;
   int          s1, s2;
   int          res1i, res2i;
   int          i;
