@@ -51,8 +51,6 @@ potts_Build(ESL_RANDOMNESS *r, ESL_MSA *msa, double ptmuh, double ptmue, PTTRAIN
   double  neff;
   int     status;
   
-  tol = 1e-6; // most time convergece is not reached, just stop at MAXIT
-  
   e2_DLogsumInit();
   
   pt = potts_Create(msa->alen, msa->abc->K+1, msa->abc, ptmuh, ptmue, pttrain, ptmin, ptsctype, ptreg, isgremlin);
@@ -113,12 +111,14 @@ potts_Build(ESL_RANDOMNESS *r, ESL_MSA *msa, double ptmuh, double ptmue, PTTRAIN
     ESL_XFAIL(eslFAIL, errbuf, "error, you should not be here");
     break;
   case PLM:
-    stol = 0.5;
+    tol  = 1e-6;   // most time convergence is not reached, just stop at MAXIT
+    stol = 1e-6;
     status = potts_Optimize_PLM(pt, msa, tol, stol, errbuf, verbose);
     if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "error all optimizing potts");
     break;
   case APLM:
-    stol = 0.5;    
+    tol  = 1e-6;   // most time convergence is not reached, just stop at MAXIT
+    stol = 1e-1;    
     status = potts_Optimize_APLM(pt, msa, tol, stol, errbuf, verbose);
     if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "error aplm optimizing potts");
     break;
@@ -220,7 +220,6 @@ potts_GaugeZeroSum(PT *pt, char *errbuf, int verbose)
   double   sum;
   int      i, j;
   int      a, b;
-  int      status;
 
   for (i = 0; i < pt->L; i++) {
     
@@ -299,9 +298,6 @@ potts_GaugeZeroSum(PT *pt, char *errbuf, int verbose)
 #endif
   
   return eslOK;
-
- ERROR:
-  return status;
 }
 
 int
@@ -311,7 +307,6 @@ potts_InitZero(PT *pt, char *errbuf, int verbose)
   int Kg = pt->Kg;
   int i, j;
   int a, b;
-  int status;
   
   for (i = 0; i < L; i++) {
     esl_vec_DSet(pt->h[i], pt->Kg, 0.0);
@@ -323,9 +318,6 @@ potts_InitZero(PT *pt, char *errbuf, int verbose)
   }
 
   return eslOK;
-
- ERROR:
-  return status;
 }
 
 int
@@ -383,7 +375,6 @@ potts_InitGremlin(ESL_RANDOMNESS *r, ESL_MSA *msa, PT *pt, double tol, char *err
   int      resi;
   int      a;
   int      status;
-  double   sum;
 
   // calculate frequecies per position pi
   ESL_ALLOC(p,    sizeof(double *)*L);
@@ -426,6 +417,54 @@ potts_InitGremlin(ESL_RANDOMNESS *r, ESL_MSA *msa, PT *pt, double tol, char *err
 
 int
 potts_InitGT(ESL_RANDOMNESS *r, ESL_MSA *msa, PT *pt, float tol, char *errbuf, int verbose)
+{
+  struct mutual_s *mi = NULL;
+  double           pseudoc = 0.01;
+  double           exp, obs;
+  double           gt;
+  int              L  = pt->L;
+  int              K  = pt->abc->K;
+  int              Kg = pt->Kg;
+  int              i, j;
+  int              a, b;
+  int              status;
+
+  mi = corr_Create(L, msa->nseq, FALSE, 0, 0, pt->abc, C16);
+  if (mi == NULL) ESL_XFAIL(eslFAIL, errbuf, "could not create mi");
+  
+  status = corr_Probs(r, msa, NULL, NULL, mi, POTTS, tol, verbose, errbuf);
+  if (status != eslOK) goto ERROR;
+  
+  for (i = 0; i < L; i++) {
+    
+    pt->h[i][K] = 0.;
+    for (a = 0; a < K; a ++) pt->h[i][a] = log(mi->pm[i][a]);
+    
+    for (j = i+1; j < L; j++) {
+      
+      for (a = 0; a < K; a ++) 
+	for (b = 0; b < K; b ++) {
+	  
+	  exp = mi->pm[i][a] * mi->pm[j][b];
+	  obs = mi->pp[i][j][IDX(a,b,K)];
+	  gt  = (exp > 0. && obs > 0.) ? log (obs / exp) : pseudoc;
+	  
+	  pt->e[i][j][IDX(a,b,Kg)] = pt->e[j][i][IDX(b,a,Kg)] = gt;
+	}
+    }
+  }
+  if (verbose) potts_Write(stdout, pt);
+  
+  corr_Destroy(mi);
+  return eslOK;
+  
+ ERROR:
+  if (mi) corr_Destroy(mi);
+  return status;
+}
+  
+int
+potts_InitGTold(ESL_RANDOMNESS *r, ESL_MSA *msa, PT *pt, float tol, char *errbuf, int verbose)
 {
   struct mutual_s *mi = NULL;
   double           exp;
@@ -448,7 +487,7 @@ potts_InitGT(ESL_RANDOMNESS *r, ESL_MSA *msa, PT *pt, float tol, char *errbuf, i
   mi = corr_Create(L, msa->nseq, FALSE, 0, 0, pt->abc, C16);
   if (mi == NULL) ESL_XFAIL(eslFAIL, errbuf, "could not create mi");
 
-  status = corr_Probs(r, msa, NULL, NULL, mi, POTTS, FALSE, tol, verbose, errbuf);
+  status = corr_Probs(r, msa, NULL, NULL, mi, POTTS, tol, verbose, errbuf);
   if (status != eslOK) goto ERROR;
   
   status = corr_CalculateGT_C16(mi, verbose, errbuf);
@@ -521,14 +560,15 @@ int
 potts_Optimize_PLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, int verbose)
 {
   struct optimize_data   data;
-  PT                   *gr   = NULL;           /* the gradient */
-  double                *p   = NULL;	       /* parameter vector                        */
-  double                *u   = NULL;           /* max initial step size vector            */
-  double                *wrk = NULL;           /* 4 tmp vectors of length nbranches       */
+  ESL_MIN_DAT          *stats = esl_min_dat_Create(NULL);
+  PT                   *gr    = NULL;           /* the gradient */
+  double                *p    = NULL;	        /* parameter vector                        */
+  double                *u    = NULL;           /* max initial step size vector            */
+  double                *wrk  = NULL;           /* 4 tmp vectors of length nbranches       */
   double                 logp;
-  int                    L   = msa->alen;
-  int                    Kg  = pt->Kg;
-  int                    Kg2 = pt->Kg2;
+  int                    L    = msa->alen;
+  int                    Kg   = pt->Kg;
+  int                    Kg2  = pt->Kg2;
   int                    np;
   int                    status;
 
@@ -559,18 +599,17 @@ potts_Optimize_PLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, in
   
   switch(pt->mintype) {
   case MINNONE:
+    status = eslOK;
       break;
   case CGD_WOLFE:
-    status = min_ConjugateGradientDescent(p, u, np,
+    status = min_ConjugateGradientDescent(NULL, p, np,
                                           &optimize_potts_func_plm, &optimize_potts_bothfunc_plm,
-                                          (void *) (&data),					   
-                                          tol, stol, wrk, &logp);
+                                          (void *) (&data), &logp, stats);
     break;
   case CGD_BRENT:
-    status = esl_min_ConjugateGradientDescent(p, u, np,
+    status = esl_min_ConjugateGradientDescent(NULL, p, np,
 					      &optimize_potts_func_plm, &optimize_potts_dfunc_plm,
-					      (void *) (&data), 
-					      tol, wrk, &logp);
+					      (void *) (&data), &logp, stats);
     break;
   case LBFGS:
     ESL_XFAIL(eslFAIL, errbuf, "LBFGS not implemented yet");
@@ -586,19 +625,23 @@ potts_Optimize_PLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, in
   if (verbose) printf("END POTTS PLM OPTIMIZATION\n");
   
   if (verbose) potts_Write(stdout, pt);
-  
+
+  if (stats) esl_min_dat_Dump(stdout, stats);
+
   /* clean up */
-  if (gr  != NULL) potts_Destroy(gr);
-  if (u   != NULL) free(u);
-  if (p   != NULL) free(p);
-  if (wrk != NULL) free(wrk);
+  if (stats) esl_min_dat_Destroy(stats);
+  if (gr) potts_Destroy(gr);
+  if (u) free(u);
+  if (p) free(p);
+  if (wrk) free(wrk);
   return eslOK;
   
  ERROR:
-  if (gr  != NULL) potts_Destroy(gr);
-  if (p   != NULL) free(p);
-  if (u   != NULL) free(u);
-  if (wrk != NULL) free(wrk);
+  if (stats) esl_min_dat_Destroy(stats);
+  if (gr) potts_Destroy(gr);
+  if (p) free(p);
+  if (u) free(u);
+  if (wrk) free(wrk);
   return status;
 }
 
@@ -607,14 +650,15 @@ int
 potts_Optimize_APLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, int verbose)
 {
   struct optimize_data  data;
-  PT                   *gr   = NULL;           /* the gradient */
-  double                *p   = NULL;	       /* parameter vector                        */
-  double                *u   = NULL;           /* max initial step size vector            */
-  double                *wrk = NULL;           /* 4 tmp vectors of length nbranches       */
+  ESL_MIN_DAT          *stats = esl_min_dat_Create(NULL);
+  PT                   *gr    = NULL;           /* the gradient */
+  double                *p    = NULL;	       /* parameter vector                        */
+  double                *u    = NULL;           /* max initial step size vector            */
+  double                *wrk  = NULL;           /* 4 tmp vectors of length nbranches       */
   double                 logp;
-  int                    L   = msa->alen;
-  int                    Kg  = pt->Kg;
-  int                    Kg2 = pt->Kg2;
+  int                    L    = msa->alen;
+  int                    Kg   = pt->Kg;
+  int                    Kg2  = pt->Kg2;
   int                    np;
   int                    i;
   int                    status;
@@ -650,16 +694,14 @@ potts_Optimize_APLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, i
     case MINNONE:
       break;
     case CGD_WOLFE:
-      status = min_ConjugateGradientDescent(p, u, np,
+      status = min_ConjugateGradientDescent(NULL, p, np,
 					    &optimize_potts_func_aplm, &optimize_potts_bothfunc_aplm,
-					    (void *) (&data),					   
-					    tol, stol, wrk, &logp);
+					    (void *) (&data), &logp, stats);
       break;
     case CGD_BRENT:
-      status = esl_min_ConjugateGradientDescent(p, u, np,
+      status = esl_min_ConjugateGradientDescent(NULL, p, np,
 						&optimize_potts_func_aplm, &optimize_potts_dfunc_aplm,
-						(void *) (&data), 
-						tol, wrk, &logp);
+						(void *) (&data), &logp, stats);
       break;
     case LBFGS:
       ESL_XFAIL(eslFAIL, errbuf, "LBFGS APLM not implemented yet");
@@ -681,18 +723,22 @@ potts_Optimize_APLM(PT *pt, ESL_MSA *msa, float tol, float stol, char *errbuf, i
   
   if (verbose) potts_Write(stdout, pt);
   
+  if (stats) esl_min_dat_Dump(stdout, stats);
+
   /* clean up */
-  if (gr  != NULL) potts_Destroy(gr);
-  if (u   != NULL) free(u);
-  if (p   != NULL) free(p);
-  if (wrk != NULL) free(wrk);
+  if (stats) esl_min_dat_Destroy(stats);
+  if (gr) potts_Destroy(gr);
+  if (u) free(u);
+  if (p) free(p);
+  if (wrk) free(wrk);
   return eslOK;
 
  ERROR:
-  if (gr  != NULL) potts_Destroy(gr);
-  if (p   != NULL) free(p);
-  if (u   != NULL) free(u);
-  if (wrk != NULL) free(wrk);
+  if (stats) esl_min_dat_Destroy(stats);
+  if (gr) potts_Destroy(gr);
+  if (p) free(p);
+  if (u) free(u);
+  if (wrk) free(wrk);
   return status;
 }
 
@@ -855,7 +901,6 @@ optimize_plm_pack_paramvector(double *p, int np, struct optimize_data *data)
 {
   int   L   = data->msa->alen;
   int   Kg  = data->pt->Kg;
-  int   Kg2 = data->pt->Kg2;
   int   x   = 0;
   int   i, j;
   int   a, b;
@@ -877,7 +922,6 @@ optimize_plm_pack_gradient(double *dx, int np, struct optimize_data *data)
 {
   int   L   = data->msa->alen;
   int   Kg  = data->pt->Kg;
-  int   Kg2 = data->pt->Kg2;
   int   x   = 0;
   int   i, j;
   int   a, b;
@@ -937,13 +981,11 @@ optimize_aplm_pack_gradient(double *dx, int np, struct optimize_data *data)
 static int
 optimize_plm_unpack_paramvector(double *p, int np, struct optimize_data *data)
 {
-  double eij, eji;
   int    L  = data->msa->alen;
   int    Kg = data->pt->Kg;
   int    x  = 0;
   int    i, j;
   int    a, b;
-  int    status;
   
   for (a = 0; a < Kg; a++)
     for (i = 0; i < L; i++) 
@@ -956,9 +998,6 @@ optimize_plm_unpack_paramvector(double *p, int np, struct optimize_data *data)
        	  data->pt->e[i][j][IDX(a,b,Kg)] = data->pt->e[j][i][IDX(b,a,Kg)] = p[x++];
   
   return eslOK;
-  
- ERROR:
-  return status;
 }
 
 static int
@@ -971,7 +1010,6 @@ optimize_aplm_unpack_paramvector(double *p, int np, struct optimize_data *data)
   int    i   = data->pos;
   int    j;
   int    a, a2;
-  int    status;
   
   for (a = 0; a < Kg; a++)       data->pt->h[i][a]     = p[x++];
   for (j = 0;   j < i; j++) 
@@ -980,9 +1018,6 @@ optimize_aplm_unpack_paramvector(double *p, int np, struct optimize_data *data)
     for (a2 = 0; a2 < Kg2; a2++) data->pt->e[i][j][a2] = p[x++];
 
   return eslOK;
-
- ERROR:
-  return status;
 }
 
 static void
